from pev import Eventer
"""
ApplicationStarted = 0

Event when application is started	

ApplicationStopped = 1
Event when application is stopped

BeforeDatabaseEngineConfiguration = 2
AfterDatabaseEngineConfiguration = 3
BeforeInitStatic = 4
AfterInitStatic = 5
BeforeApplicationStarted = 6
BeforeInitServer = 7
AfterInitServer = 8

BeforeControllerAction = 2
Event before controller action will be executed.
You can change Request object before it will pass to controller action.

AfterControllerAction = 3
Event after controller action was executed.
You can change Response object before rendering.
"""
